import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Objects;

import org.apache.commons.compress.archivers.sevenz.SevenZArchiveEntry;
import org.apache.commons.compress.archivers.sevenz.SevenZOutputFile;
import org.jetel.component.AbstractGenericTransform;
import org.jetel.data.DataField;
import org.jetel.data.DataRecord;
import org.jetel.util.property.RefResFlag;
import org.jetel.util.stream.StreamUtils;
import org.jetel.util.string.StringUtils;

/**
 * This is an example custom reader. It shows how you can
 *  create records using a data source.
 */
public class SevenZipWriter extends AbstractGenericTransform {
	
	private String fileUrl;

	private DataRecord record;
	private DataField fName;
	private DataField fURL;
	
	private static String toString(Object o) {
		return Objects.toString(o, null);
	}
	
	@Override
	public void execute() throws Exception {
		File outputFile = getFile(fileUrl);
		
		if (outputFile == null) {
			throw new FileNotFoundException(fileUrl);
		}
		
		try (SevenZOutputFile zipFile = new SevenZOutputFile(outputFile)) {
			while ((record = readRecordFromPort(0)) != null) {
				String entryName = toString(fName.getValue());
				String url = toString(fURL.getValue());
				
				File inputFile = getFile(url);
				if (inputFile == null) {
					throw new FileNotFoundException(url);
				}
				
				if (StringUtils.isEmpty(entryName)) {
					entryName = inputFile.getName();
				}
				SevenZArchiveEntry entry = zipFile.createArchiveEntry(inputFile, entryName);
				zipFile.putArchiveEntry(entry);
				if (inputFile.isFile()) {
					try (
						InputStream is = new FileInputStream(inputFile);
						OutputStream os = new SevenZipOutputStream(zipFile);
					) {
						StreamUtils.copy(is, os);
					}
				}
				zipFile.closeArchiveEntry();
			}
		}

	}

	@Override
	public void init() {
		super.init();

		fileUrl = getProperties().getStringProperty("fileUrl", null, RefResFlag.URL);
		
		record = inRecords[0];
		fName = record.getField("entryName");
		fURL = record.getField("URL");
	}
	
	private static class SevenZipOutputStream extends OutputStream {
		
		private final SevenZOutputFile file;

		public SevenZipOutputStream(SevenZOutputFile file) {
			this.file = file;
		}

		@Override
		public void write(int b) throws IOException {
			file.write(b);
		}

		@Override
		public void write(byte[] b, int off, int len) throws IOException {
			file.write(b, off, len);
		}
		
	}

}
